/*global $, DataProviderHelper, SessionTimeoutError, ServerPermissionError, ErrorUtility */

var DataProvider = {};
(function () {
    'use strict';
    var context = window.parent;

    DataProvider.validateResponse = function validateResponse(xhr) {
        var error = DataProviderHelper.validateRequest(xhr);
        if (!error) {
            var responseText = xhr.responseText;
            if (typeof responseText === 'string') {
                if (responseText.indexOf('id="user-login-form"') !== -1 ||
                    responseText.indexOf('id="user-login"') !== -1) {
                    error = new SessionTimeoutError();
                    error.loginUrl = context.location.href;
                } else {
                    try {
                        var obj = JSON.parse(responseText);
                        if (obj.result === 'fail' && obj.type === 'permission') {
                            error = new ServerPermissionError(obj.error);
                        }
                    } catch (e) {
                    }
                }
            }
        }
        return error;
    };

    function ajaxFailHandler(url, xhr, status, callback) {
        var error = DataProvider.validateResponse(xhr);
        if (!error) {
            error = ErrorUtility.createRequestError(url, xhr, status, 'Request fail');
        }
        callback(error);
    }

    function doRequest(data, onError, onSuccess) {
        if (!onError || typeof onError !== 'function') {
            throw DataProviderHelper.getResultError('Invalid callback');
        }

        $.ajax({
            url: window.pluginData.url.process_request,
            type: 'post',
            dataType: 'json',
            data: jQuery.extend({
                uid: window.pluginData.uid
            }, data)
        }).done(function requestSuccess(response, status, xhr) {
            var error = DataProvider.validateResponse(xhr);
            if (error) {
                onError(error);
            } else if (response.result === 'done') {
                if (typeof onSuccess === 'undefined') {
                    onError(error, {logs: response.log || []});
                } else {
                    onSuccess(response);
                }
            } else {
                var invalidResponseError = ErrorUtility.createRequestError(window.pluginData.url.process_request, xhr, status, (data.action || '') + ' server error: ' + JSON.stringify(response));
                onError(invalidResponseError);
            }
        }).fail(function requestFail(xhr, status) {
            ajaxFailHandler(window.pluginData.url.process_request, xhr, status, onError);
        });
    }

    function reloadInfo(action, callback) {
        if (!callback || typeof callback !== 'function') {
            throw DataProviderHelper.getResultError('Invalid callback');
        }

        $.ajax({
            url: window.pluginData.url.process_request,
            type: 'post',
            data: ({
                action : action,
                uid: window.pluginData.uid,
                themeName: window.pluginData.themeName
            })
        }).done(function reloadInfo(data, status, xhr) {
            var error = DataProvider.validateResponse(xhr);
            if (!error) {
                try {
                    var info = JSON.parse(data);
                    $.each(info, function(key, value) {
                        window.pluginData[key] = value;
                    });
                } catch(e) {
                    error = new Error(e);
                    error.args = { parseArgument: data };
                }
            } 
            callback(error, data);
        }).fail(function reloadInfo(xhr, status) {
            ajaxFailHandler(window.pluginData.ajax, xhr, status, callback);
        });
    };

    DataProvider.backToAdmin = function backToAdmin() {
        window.location = window.pluginData.url.admin;
    };

    DataProvider.getMaxRequestSize = function getMaxRequestSize() {
        return window.pluginData.maxRequestSize;
    };

    DataProvider.getAllCssJsSources = function getAllCssJsSources() {
        return window.pluginData.cssJsSources;
    };

    DataProvider.getMd5Hashes = function getMd5Hashes() {
        return window.pluginData.md5Hashes;
    };

    DataProvider.getTheme = function getTheme(data, callback) {
        $.ajax({
            url: window.pluginData.url.process_request,
            type: 'post',
            dataType: 'text',
            data: ({
                action: 'download',
                uid: window.pluginData.uid,
                themeName: window.pluginData.themeName,
                data: data
            })
        }).done(function getThemeSuccess(response, status, xhr) {
            var error = DataProvider.validateResponse(xhr);
            if (!error) {
                callback(null, response);
            } else {
                callback(error);
            }
        }).fail(function getThemeFail(xhr, status) {
            ajaxFailHandler(window.pluginData.url.process_request, xhr, status, callback);
        });
    };

    DataProvider.downloadTheme = function getTheme(id, callback) {
        DataProvider.getTheme({id: id}, callback);
    };

    DataProvider.doExport = function doExport(exportData, callback) {
        var request = {
            'save': {
                'post': {
                    data: JSON.stringify(exportData),
                    action : 'export',
                    uid: window.pluginData.uid,
                    themeName: window.pluginData.themeName
                },
                'url': window.pluginData.url.process_request
            },
            'clear': {
                'post': {
                    action : 'clear',
                    uid: window.pluginData.uid
                },
                'url': window.pluginData.url.process_request
            },
            'errorHandler': DataProvider.validateResponse,
            'encode': true,
            'blob': true
        };
        DataProviderHelper.chunkedRequest(request, callback);
    };

    DataProvider.save = function (saveData, callback) {
        var request = {
            'save': {
                'post': {
                    action: 'save',
                    data: JSON.stringify(saveData),
                    uid: window.pluginData.uid,
                    themeName: window.pluginData.themeName,
                    isNew: window.pluginData.isNew
                },
                'url': window.pluginData.url.process_request
            },
            'clear': {
                'post': {
                    action : 'clear',
                    uid: window.pluginData.uid
                },
                'url': window.pluginData.url.process_request
            },
            'errorHandler': DataProvider.validateResponse,
            'encode': true,
            'blob': true
        };
        DataProviderHelper.chunkedRequest(request, callback);
    };

    DataProvider.load = function() {
        return window.pluginData.project.projectData;
    };

    DataProvider.updatePreviewTheme = function updatePreviewTheme(callback) {
        doRequest({action: 'update', themeName: window.pluginData.themeName}, callback);
    }

    DataProvider.makeThemeAsActive = function makeThemeAsActive(callback, id) {
        var data = {action: 'activate', themeName: window.pluginData.themeName};
        if (typeof id !== 'undefined') {
            data.newName = id;
        }
        doRequest(data, callback);
    };

    DataProvider.removeTheme = function removeTheme(id, callback) {
        doRequest({action: 'remove', removedName: id}, callback);
    };

    DataProvider.copyTheme = function copyTheme(id, newName, callback) {
        var data = {action: 'copy', themeName: id};
        if (typeof newName !== 'undefined') {
            data.newName = newName;
        }
        doRequest(data, callback);
    };

    DataProvider.renameTheme = function renameTheme(themeName, newName, callback) {
        doRequest({action: 'rename', themeName: themeName, newName: newName}, callback, function(response) {
            callback(null, context.location.href.replace(new RegExp('theme=' + window.pluginData.themeName), 'theme=' + newName));
        });
    };

    DataProvider.rename = function rename(newName, callback) {
        DataProvider.renameTheme(window.pluginData.themeName, newName, callback);
    };

    DataProvider.canRename = function canRename(themeName, callback) {
        doRequest({action: 'canrename', themeName: window.pluginData.themeName, newName: themeName}, callback, function(response) {
            if (response.message === '') {
                callback(null, true);
            } else {
                callback(null, false, response.message);
            }
        });
    };

    DataProvider.themeNameValidation = function validation(name, callback) {
        doRequest({action: 'validation', themeName: window.pluginData.themeName, newName: name}, callback, function(response) {
            if (response.message === '') {
                callback(null);
            } else {
                callback(null, response.message);
            }
        });
    };

    DataProvider.getFiles = function getFiles(mask, filter, callback) {
        doRequest({action: 'get_files', themeName: window.pluginData.themeName, mask: mask || '*', filter: filter || ''}, callback, function(response) {
            callback(null, response.files);
        });
    };

    DataProvider.setFiles = function setFiles(files, callback) {
        if (!callback || typeof callback !== 'function') {
            throw DataProviderHelper.getResultError('Invalid callback');
        }

        var request = {
            'save': {
                'post': {
                    action : 'set_files',
                    data: JSON.stringify(files),
                    uid: window.pluginData.uid,
                    themeName: window.pluginData.themeName
                },
                'url': window.pluginData.url.process_request
            },
            'clear': {
                'post': {
                    action : 'clear',
                    uid: window.pluginData.uid
                },
                'url': window.pluginData.url.process_request
            },
            'errorHandler': DataProvider.validateResponse,
            'encode': true,
            'blob': true
        };

        DataProviderHelper.chunkedRequest(request, callback);
    };

    DataProvider.reloadTemplatesInfo = function reloadTemplatesInfo(callback) {
        if (!callback || typeof callback !== 'function') {
            throw DataProviderHelper.getResultError('Invalid callback');
        }

        $.ajax({
            url: window.pluginData.url.process_request,
            type: 'post',
            data: ({
                action : 'refresh_templates',
                uid: window.pluginData.uid,
                themeName: window.pluginData.themeName
            })
        }).done(function reloadTemplatesInfoSuccess(data, status, xhr) {
            var error = DataProvider.validateResponse(xhr);
            if (!error) {
                try {
                    $.each(data, function(key, value) {
                        window.pluginData.templates[key] = value;
                    });
                } catch(e) {
                    error = new Error(e);
                    error.args = { parseArgument: data };
                }
            } 
            callback(error);
        }).fail(function reloadTemplatesInfoFail(xhr, status) {
            ajaxFailHandler(window.pluginData.url.process_request, xhr, status, callback);
        });
    };

    DataProvider.reloadTemplatesInfo = function reloadTemplatesInfo(callback) {
        reloadInfo('refresh_templates', callback);
    }
    
    DataProvider.reloadThemesInfo = function reloadTemplatesInfo(callback) {
        reloadInfo('refresh_themes', callback);
    }
    
    DataProvider.getInfo = function getInfo(){
        return {
            cmsName: 'Drupal',
            cmsVersion: window.pluginData.cmsVersion,
            adminPage: window.pluginData.url.admin,
            startPage: window.pluginData.url.preview,
            uploadImage: window.pluginData.url.upload_image,
            uploadTheme: window.pluginData.url.upload_theme,
            templates: window.pluginData.templates,
            themes: window.pluginData.themes,
            thumbnails: [{name: 'screenshot.png', width: 300, height: 220, version: '7.x'},{name: 'screenshot.png', width: 150, height: 90, version: '6.x'}],
            isThemeActive: window.pluginData.isActive,
            themeName: window.pluginData.themeName,
            pathToManifest: '/export/themler.manifest'
        };
    };

    DataProvider.getVersion = function () {
        return "0.0.2";
    };

}());